@@ -90,8 +90,8 @@ _base_info_richcompare (PyGIBaseInfo *self, PyObject *other, int op)
 
 PYGLIB_DEFINE_TYPE("gi.BaseInfo", PyGIBaseInfo_Type, PyGIBaseInfo);
 
-static PyObject *
-_wrap_g_base_info_get_name (PyGIBaseInfo *self)
+gboolean
+_pygi_is_python_keyword (const gchar *name)
 {
     /* It may be better to use keyword.iskeyword(); keep in sync with
      * python -c 'import keyword; print(keyword.kwlist)' */
@@ -115,18 +115,30 @@ _wrap_g_base_info_get_name (PyGIBaseInfo *self)
     #error Need keyword list for this major Python version
 #endif
 
-    const gchar *name, **i;
+    const gchar **i;
+
+    for (i = keywords; *i != NULL; ++i) {
+        if (strcmp (name, *i) == 0) {
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+static PyObject *
+_wrap_g_base_info_get_name (PyGIBaseInfo *self)
+{
+    const gchar *name;
 
     name = g_base_info_get_name (self->info);
 
     /* escape keywords */
-    for (i = keywords; *i != NULL; ++i) {
-        if (strcmp (name, *i) == 0) {
-            gchar *escaped = g_strconcat (name, "_", NULL);
-            PyObject *obj = PYGLIB_PyUnicode_FromString (escaped);
-            g_free (escaped);
-            return obj;
-        }
+    if (_pygi_is_python_keyword (name)) {
+        gchar *escaped = g_strconcat (name, "_", NULL);
+        PyObject *obj = PYGLIB_PyUnicode_FromString (escaped);
+        g_free (escaped);
+        return obj;
     }
 
     return PYGLIB_PyUnicode_FromString (name);
@@ -419,11 +431,19 @@ _g_arg_get_pytype_hint (PyGIBaseInfo *self)
     } else {
 	Py_DecRef(py_type);
 	if (type_tag == GI_TYPE_TAG_INTERFACE) {
+	    const char *info_name;
+	    PyObject *py_string;
 	    GIBaseInfo *iface = g_type_info_get_interface(&type_info);
+
+	    info_name = g_base_info_get_name (iface);
+	    if (info_name == NULL) {
+	        g_base_info_unref (iface);
+	        return PYGLIB_PyUnicode_FromString(g_type_tag_to_string(type_tag));
+	    }
+
 	    gchar *name = g_strdup_printf("%s.%s",
 		    g_base_info_get_namespace(iface),
-		    g_base_info_get_name (iface));
-        PyObject *py_string;
+		    info_name);
 	    g_base_info_unref(iface);
 	    py_string = PYGLIB_PyUnicode_FromString(name);
 	    g_free(name);
